@@ -1,6 +1,7 @@
-/*	$Id: lukemftp.h,v 1.12 1999/10/27 06:56:26 lukem Exp $	*/
+/* $Id: lukemftp.h,v 1.13 1999/11/13 01:40:40 lukem Exp $ */
 
-#define	VERSION	"lukemftp 1.0"
+#define	FTP_PRODUCT	"lukemftp"
+#define	FTP_VERSION	"1.1"
 
 #include "config.h"
 
@@ -17,7 +18,6 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <sys/utsname.h>
 #include <sys/wait.h>
 
 #include <netinet/in.h>
