@@ -1,4 +1,4 @@
-/*	$NetBSD: cmds.c,v 1.76 1999/10/24 12:31:36 lukem Exp $	*/
+/*	$NetBSD: cmds.c,v 1.79 1999/11/11 02:53:02 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-1999 The NetBSD Foundation, Inc.
@@ -740,7 +740,6 @@ status(argc, argv)
 		code = -1;
 		return;
 	}
-	fprintf(ttyout, "%s version: %s\n", __progname, VERSION);
 	if (connected)
 		fprintf(ttyout, "Connected %sto %s.\n",
 		    connected == -1 ? "and logged in" : "", hostname);
@@ -804,6 +803,7 @@ status(argc, argv)
 	    onoff(editing)
 #endif	/* !def NO_EDITCOMPLETE */
 	    );
+	fprintf(ttyout, "Version: %s %s\n", FTP_PRODUCT, FTP_VERSION);
 	if (macnum > 0) {
 		fputs("Macros:\n", ttyout);
 		for (i=0; i<macnum; i++) {
@@ -1118,8 +1118,10 @@ cd(argc, argv)
 			    ttyout);
 		r = command("XCWD %s", argv[1]);
 	}
-	if (r == COMPLETE)
+	if (r == COMPLETE) {
 		dirchange = 1;
+		updateremotepwd();
+	}
 }
 
 /*
@@ -2328,8 +2330,10 @@ cdup(argc, argv)
 			    ttyout);
 		r = command("XCUP");
 	}
-	if (r == COMPLETE)
+	if (r == COMPLETE) {
 		dirchange = 1;
+		updateremotepwd();
+	}
 }
 
 /*
