@@ -1,4 +1,4 @@
-/*	$NetBSD: main.c,v 1.65 1999/10/24 12:31:41 lukem Exp $	*/
+/*	$NetBSD: main.c,v 1.67 1999/11/12 02:50:38 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-1999 The NetBSD Foundation, Inc.
@@ -396,11 +396,19 @@ main(argc, argv)
 	(void)strlcpy(anonpass, anonuser, len);
 	(void)strlcat(anonpass, "@",	  len);
 
+			/*
+			 * set all the defaults for options defined in
+			 * struct option optiontab[]  declared in cmdtab.c
+			 */
 	setupoption("anonpass",		getenv("FTPANONPASS"),	anonpass);
-	setupoption("ftp_proxy",	getenv(FTP_PROXY),	xstrdup(""));
-	setupoption("http_proxy",	getenv(HTTP_PROXY),	xstrdup(""));
-	setupoption("no_proxy",		getenv(NO_PROXY),	xstrdup(""));
-	setupoption("pager",		getenv("PAGER"), xstrdup(DEFAULTPAGER));
+	setupoption("ftp_proxy",	getenv(FTP_PROXY),	"");
+	setupoption("http_proxy",	getenv(HTTP_PROXY),	"");
+	setupoption("no_proxy",		getenv(NO_PROXY),	"");
+	setupoption("pager",		getenv("PAGER"),	DEFAULTPAGER);
+	setupoption("prompt",		getenv("FTPPROMPT"),	DEFAULTPROMPT);
+	setupoption("rprompt",		getenv("FTPRPROMPT"),	DEFAULTRPROMPT);
+
+	free(anonpass);
 
 	setttywidth(0);
 #ifdef SIGINFO
@@ -451,6 +459,7 @@ main(argc, argv)
 #ifndef NO_EDITCOMPLETE
 	controlediting();
 #endif /* !NO_EDITCOMPLETE */
+
 	(void)sigsetjmp(toplevel, 1);
 	(void)xsignal(SIGINT, intr);
 	(void)xsignal(SIGPIPE, lostpeer);
@@ -464,7 +473,40 @@ main(argc, argv)
 char *
 prompt()
 {
-	return ("ftp> ");
+	static char	**prompt;
+	static char	  buf[MAXPATHLEN];
+
+	if (prompt == NULL) {
+		struct option *o;
+
+		o = getoption("prompt");
+		if (o == NULL)
+			errx(1, "no such option `prompt'");
+		prompt = &(o->value);
+	}
+	formatbuf(buf, sizeof(buf), *prompt ? *prompt : DEFAULTPROMPT);
+	return (buf);
+}
+
+/*
+ * Generate an rprompt
+ */
+char *
+rprompt()
+{
+	static char	**rprompt;
+	static char	  buf[MAXPATHLEN];
+
+	if (rprompt == NULL) {
+		struct option *o;
+
+		o = getoption("rprompt");
+		if (o == NULL)
+			errx(1, "no such option `rprompt'");
+		rprompt = &(o->value);
+	}
+	formatbuf(buf, sizeof(buf), *rprompt ? *rprompt : DEFAULTRPROMPT);
+	return (buf);
 }
 
 /*
@@ -473,8 +515,9 @@ prompt()
 void
 cmdscanner()
 {
-	struct cmd *c;
-	int num;
+	struct cmd	*c;
+	char		*p;
+	int		 num;
 
 	for (;;) {
 #ifndef NO_EDITCOMPLETE
@@ -482,6 +525,9 @@ cmdscanner()
 #endif /* !NO_EDITCOMPLETE */
 			if (fromatty) {
 				fputs(prompt(), ttyout);
+				p = rprompt();
+				if (*p)
+					fprintf(ttyout, "%s ", p);
 				(void)fflush(ttyout);
 			}
 			if (fgets(line, sizeof(line), stdin) == NULL) {
@@ -783,12 +829,13 @@ help(argc, argv)
 	char *argv[];
 {
 	struct cmd *c;
-	char *nargv[1], *p;
+	char *nargv[1], *p, *cmd;
 	int isusage;
 
-	isusage = (strcmp(argv[0], "usage") == 0);
+	cmd = argv[0];
+	isusage = (strcmp(cmd, "usage") == 0);
 	if (argc == 0 || (isusage && argc == 1)) {
-		fprintf(ttyout, "usage: %s [command [...]]\n", argv[0]);
+		fprintf(ttyout, "usage: %s [command [...]]\n", cmd);
 		return;
 	}
 	if (argc == 1) {
@@ -815,10 +862,10 @@ help(argc, argv)
 		c = getcmd(arg);
 		if (c == (struct cmd *)-1)
 			fprintf(ttyout, "?Ambiguous %s command `%s'\n",
-			    argv[0], arg);
+			    cmd, arg);
 		else if (c == NULL)
 			fprintf(ttyout, "?Invalid %s command `%s'\n",
-			    argv[0], arg);
+			    cmd, arg);
 		else {
 			if (isusage) {
 				nargv[0] = arg;
@@ -850,15 +897,13 @@ char *
 getoptionvalue(name)
 	const char *name;
 {
-	const char *p;
 	struct option *c;
 
 	if (name == NULL)
 		errx(1, "getoptionvalue() invoked with NULL name");
-	for (c = optiontab; (p = c->name) != NULL; c++) {
-		if (strcasecmp(p, name) == 0)
-			return (c->value);
-	}
+	c = getoption(name);
+	if (c != NULL)
+		return (c->value);
 	errx(1, "getoptionvalue() invoked with unknown option `%s'", name);
 }
 
