@@ -1,4 +1,4 @@
-/*	$NetBSD: util.c,v 1.79 1999/10/24 12:31:42 lukem Exp $	*/
+/*	$NetBSD: util.c,v 1.82 1999/11/12 02:50:38 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1997-1999 The NetBSD Foundation, Inc.
@@ -114,7 +114,7 @@ setpeer(argc, argv)
 	else
 		port = ftpport;
 	if (argc > 2)
-		port = strdup(argv[2]);
+		port = argv[2];
 
 	if (gatemode) {
 		if (gateserver == NULL || *gateserver == '\0')
@@ -199,6 +199,7 @@ setpeer(argc, argv)
 "Remember to set tenex mode when transferring binary files from this machine.\n",
 				    ttyout);
 		}
+		updateremotepwd();
 		verbose = overbose;
 	}
 }
@@ -226,6 +227,8 @@ cleanuppeer()
 		anonftp = 0;
 	data = -1;
 	epsv4bad = 0;
+	if (username)
+		free(username);
 	if (!proxy)
 		macnum = 0;
 }
@@ -381,6 +384,7 @@ ftp_login(host, user, pass)
 		goto cleanup_ftp_login;
 	}
 	rval = 1;
+	username = xstrdup(user);
 	if (proxy)
 		goto cleanup_ftp_login;
 
@@ -633,6 +637,45 @@ remotemodtime(file, noisy)
 	return (rtime);
 }
 
+/*
+ * update global `remotepwd', which contains the state of the remote cwd
+ */
+void
+updateremotepwd()
+{
+	int	 overbose, ocode, i;
+	char	*cp;
+
+	overbose = verbose;
+	ocode = code;
+	if (debug == 0)
+		verbose = -1;
+	if (command("PWD") != COMPLETE)
+		goto badremotepwd;
+	cp = strchr(reply_string, ' ');
+	if (cp == NULL || cp[0] == '\0' || cp[1] != '"')
+		goto badremotepwd;
+	cp += 2;
+	for (i = 0; *cp && i < sizeof(remotepwd) - 1; i++, cp++) {
+		if (cp[0] == '"') {
+			if (cp[1] == '"')
+				cp++;
+			else
+				break;
+		}
+		remotepwd[i] = *cp;
+	}
+	remotepwd[i] = '\0';
+	if (debug)
+		fprintf(ttyout, "got remotepwd as `%s'\n", remotepwd);
+	goto cleanupremotepwd;
+ badremotepwd:
+	remotepwd[0]='\0';
+ cleanupremotepwd:
+	verbose = overbose;
+	code = ocode;
+}
+
 #ifndef	NO_PROGRESS
 
 /*
@@ -1053,7 +1096,8 @@ controlediting()
 		el_set(el, EL_HIST, history, hist);	/* use history */
 
 		el_set(el, EL_EDITOR, "emacs");	/* default editor is emacs */
-		el_set(el, EL_PROMPT, prompt);	/* set the prompt function */
+		el_set(el, EL_PROMPT, prompt);	/* set the prompt functions */
+		el_set(el, EL_RPROMPT, rprompt);
 
 		/* add local file completion, bind to TAB */
 		el_set(el, EL_ADDFN, "ftp-complete",
@@ -1172,6 +1216,122 @@ ftpvis(dst, dstlen, src, srclen)
 	dst[di] = '\0';
 }
 
+/*
+ * Copy src into buf (which is len bytes long), expanding % sequences.
+ */
+void
+formatbuf(buf, len, src)
+	char		*buf;
+	size_t		 len;
+	const char	*src;
+{
+	const char	*p;
+	char		*p2, *q;
+	int		 i, op, updirs, pdirs;
+
+#define ADDBUF(x) do { \
+		if (i >= len - 1) \
+			goto endbuf; \
+		buf[i++] = (x); \
+	} while (0)
+
+	p = src;
+	for (i = 0; *p; p++) {
+		if (*p != '%') {
+			ADDBUF(*p);
+			continue;
+		}
+		p++;
+
+		switch (op = *p) {
+
+		case '/':
+		case '.':
+		case 'c':
+			p2 = connected ? remotepwd : "";
+			updirs = pdirs = 0;
+
+			/* option to determine fixed # of dirs from path */
+			if (op == '.' || op == 'c') {
+				int skip;
+
+				q = p2;
+				while (*p2)		/* calc # of /'s */
+					if (*p2++ == '/')
+						updirs++;
+				if (p[1] == '0') {	/* print <x> or ... */
+					pdirs = 1;
+					p++;
+				}
+				if (p[1] >= '1' && p[1] <= '9') {
+							/* calc # to skip  */
+					skip = p[1] - '0';
+					p++;
+				} else
+					skip = 1;
+
+				updirs -= skip;
+				while (skip-- > 0) {
+					while ((p2 > q) && (*p2 != '/'))
+						p2--;	/* back up */
+					if (skip && p2 > q)
+						p2--;
+				}
+				if (*p2 == '/' && p2 != q)
+					p2++;
+			}
+
+			if (updirs > 0 && pdirs) {
+				if (i >= len - 5)
+					break;
+				if (op == '.') {
+					ADDBUF('.');
+					ADDBUF('.');
+					ADDBUF('.');
+				} else {
+					ADDBUF('/');
+					ADDBUF('<');
+					if (updirs > 9) {
+						ADDBUF('9');
+						ADDBUF('+');
+					} else
+						ADDBUF('0' + updirs);
+					ADDBUF('>');
+				}
+			}
+			for (; *p2; p2++)
+				ADDBUF(*p2);
+			break;
+
+		case 'M':
+		case 'm':
+			for (p2 = connected ? hostname : "-"; *p2; p2++) {
+				if (op == 'm' && *p2 == '.')
+					break;
+				ADDBUF(*p2);
+			}
+			break;
+
+		case 'n':
+			for (p2 = connected ? username : "-"; *p2 ; p2++)
+				ADDBUF(*p2);
+			break;
+
+		case '%':
+			ADDBUF('%');
+			break;
+
+		default:		/* display unknown codes literally */
+			ADDBUF('%');
+			ADDBUF(op);
+			break;
+
+		}
+	}
+ endbuf:
+	buf[i] = '\0';
+}
+
 /*
  * Determine if given string is an IPv6 address or not.
  * Return 1 for yes, 0 for no
